#!/bin/sh
# $Id: rsnap.sh 24 2016-03-19 23:07:03Z umaxx $
# Copyright (c) 2016 Joerg Jung <mail@umaxx.net>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

PATH=$PATH:/usr/local/bin

readonly R_V="0.3" R_YR="2016"
readonly R_EXCL="--exclude=/a/* --exclude=/altroot/* --exclude=/mnt/* \
                 --exclude=/proc/* --exclude=/sys/* --exclude=/tmp_mnt/*"

r_ex() {
    rm -f "${1}/.lock/err"; rmdir "${1}/.lock"
}

r_err() {
    logger -i -p user.err -s -t "$(basename ${0})" "error: ${1}"; exit 1
}

r_run() {
    local d=$(date "+%Y%m%dT%H%M%S")
    eval rsync -ahqzHS --delete --delete-excluded --link-dest=../latest \
               --numeric-ids --partial "${R_EXCL}" "${3}" \
               "${1}" "${2}/partial" 2> "${2}/.lock/err"
    [ $? -ne 0 -a $? -ne 24 ] &&
        { cat "${2}/.lock/err" >&2; r_err "rsync failed"; }
    cd "${2}"
    mv partial $d || r_err "mv failed"
    ln -fhs $d latest || r_err "ln failed"
    touch $d || r_err "touch failed"
}

r_main() {
    [ $# -eq 1 -a "${1}" == "version" ] &&
        { printf "rsnap ${R_V} (c) ${R_YR} Joerg Jung\n"; exit 0; }
    [ $# -lt 2 -o $# -gt 3 ] &&
        { printf "Usage: $0 <src> <dest> [<args>]\n%7s$0 version\n"; exit 1; }
    [ -d "${2}" ] || { mkdir -p "${2}" || r_err "mkdir failed"; }
    mkdir "${2}/.lock" && trap "r_ex \"${2}\"" 0 || r_err "lock failed"
    trap "r_err \"signal received\"" 1 2 3 13 15
    r_run "$@"
}

r_main "$@"
